import java.io.*;

/**
* A class to communicate and control the real robot arm
* @author Ho Ng
* @version 01/09/1998
*/
public class ArmController extends CRSPlusA150
{
    private Armtty connection = new Armtty();

    /**
    * purpose : move all joint angles of the arm
    */
    private void moveJoints()
    {
        double theta2 = getTheta(2);
        double theta3 = theta2 + getTheta(3) ;
        double theta4 = theta3 + getTheta(4) ;

        System.out.println("Moving arm to waist " + getTheta(1) + " " + theta2 + " " +
                theta3 + " " + theta4 + " " + getTheta(5));
        connection.ma(getTheta(1), theta2, theta3, theta4, getTheta(5));
    }

    /**
    * Default constructor
    * purpose : initialize the controller
    */
    public ArmController()
    {
        super();
    }

    /**
    * purpose : establish the connection to the arm
    * @return true (succeed) or false (fail)
    */
    public boolean connect()
    {
        if (connection.connect() == false)
            return false;

        connection.speed(30);
        return true;
    }

    /**
    * purpose : close the connection to the arm
    * @return true (succeed) or false (fail)
    */
    public boolean disconnect()
    {
        return (connection.disconnect());
    }

    /**
    * purpose : execute the command HOME of the arm
    */
    public void home()
    {
        super.home();
        connection.home();
    }

    /**
    * purpose : execute the command READY of the arm
    */
    public void ready()
    {
        super.ready();
        connection.ready();
    }
    
    /**
    * purpose : wait for the previous motion instruction to complete
    */
    public void finish()
    {
       connection.finish();
    }

    /**
    * purpose : execute the command MA of the arm
    * @param waist waist angle in radian
    * @param shoulder shoulder angle in radian
    * @param elbow elbow angle in radian
    * @param wrist wrist's bend angle in radian
    * @param twist wrist's twist angle in radian
    * @return true (succeed) or false (fail)
    */
    public boolean ma(double waist, double shoulder, double elbow,
                      double wrist, double twist)
    {
        if (super.ma(waist, shoulder, elbow, wrist, twist) == false)
            return false;

        moveJoints();
        return true;
    }

    /**
    * purpose : execute the command MA of the arm
    * @param waist waist angle in degree
    * @param shoulder shoulder angle in degree
    * @param elbow elbow angle in degree
    * @param wrist wrist's bend angle in degree
    * @param twist wrist's twist angle in degree
    * @return true (succeed) or false (fail)
    */
    public boolean madeg(double waist, double shoulder, double elbow,
                         double wrist, double twist)
    {
        if (super.madeg(waist, shoulder, elbow, wrist, twist) == false)
            return false;

        moveJoints();
        return true;
    }

    /**
    * purpose : execute the command MI of the arm
    * @param waist increment of waist angle in radian
    * @param shoulder increment of shoulder angle in radian
    * @param elbow increment of elbow angle in radian
    * @param wrist increment of wrist's bend angle in radian
    * @param twist increment of wrist's twist angle in radian
    * @return true (succeed) or false (fail)
    */
    public boolean mi(double waist, double shoulder, double elbow,
                      double wrist, double twist)
    {
        if (super.mi(waist, shoulder, elbow, wrist, twist) == false)
            return false;

        moveJoints();
        return true;
    }

    /**
    * purpose : execute the command MI of the arm
    * @param waist increment of waist angle in degree
    * @param shoulder increment of shoulder angle in degree
    * @param elbow increment of elbow angle in degree
    * @param wrist increment of wrist's bend angle in degree
    * @param twist increment of wrist's twist angle in degree
    * @return true (succeed) or false (fail)
    */
    public boolean mideg(double waist, double shoulder, double elbow,
                         double wrist, double twist)
    {
        if (super.mideg(waist, shoulder, elbow, wrist, twist) == false)
            return false;

        moveJoints();
        return true;
    }

    /**
    * to be implemented
    */
    public void move(double x0, double y0, double z0)
    {
        // to be implemented
    }

    /**
    * purpose : execute the command GRIP of the arm
    * @param width width between the fingers of the gripper in inch
    * @return true (succeed) or false (fail)
    */
    public boolean grip(double width)
    {
        if (super.grip(width) == false)
            return false;

        connection.grip(width);
        return true;
    }

    /**
    * purpose : execute the command SPEED of the arm
    * @param value speed of the arm
    * @return true (succeed) or false (fail)
    */
    public boolean speed(int value)
    {
        if (super.speed(value) == false)
            return false;

        connection.speed(value);
        return true;
    }
};
