import java.awt.event.*;
import java.awt.*;

/**
* A class to display the scene of the robot arm on a window
* @author Ho Ng
* @version 01/09/1998
*/
public class ArmDisplay extends Frame
{
    private static final int DEFAULTUSIZE = 300;
    private static final int DEFAULTVSIZE = 300;
    private static final int BUTTONWIDTH  = 100;
    private static final int BUTTONHEIGHT = 50;

    private static double CAMERARADIUS     = 62.0;
    private static double CAMERAHEIGHT     = 15.0;
    private static double CAMERAVERTICAL   = 4.0 * Math.PI / 9.0;
    private static double CAMERAHORIZONTAL = Math.PI;
    private static double CAMERASTEP       = Math.PI / 36.0;

    private Pic3DCanvas output;
    private Button button1, button2, button3, button4, button5;
    private double vertical, horizontal;

    class B1 implements ActionListener
    {
        public void actionPerformed(ActionEvent evt)
        {
            if (updateCamera(CAMERASTEP, 0.0) == true)
                output.display();
        }
    };

    class B2 implements ActionListener
    {
        public void actionPerformed(ActionEvent evt)
        {
            if (updateCamera(-CAMERASTEP, 0.0) == true)
                output.display();
        }
    };

    class B3 implements ActionListener
    {
        public void actionPerformed(ActionEvent evt)
        {
            if (updateCamera(0.0, CAMERASTEP) == true)
                output.display();
        }
    };

    class B4 implements ActionListener
    {
        public void actionPerformed(ActionEvent evt)
        {
            if (updateCamera(0.0, -CAMERASTEP) == true)
                output.display();
        }
    };

    class B5 implements ActionListener
    {
        public void actionPerformed(ActionEvent evt)
        {
            setVisible(false);
            dispose();
	    System.exit(0); // die
        }
    };

    /**
    * purpose : move the camera to other position
    * @param dv increment vertical angle by dv in radian
    * @param dh increment horizontal angle by dh in radian
    * @return true (succeed) or false (fail)
    */
    private boolean updateCamera(double dv, double dh)
    {
        vertical   += dv;
        horizontal += dh;

        if ((vertical > CAMERAVERTICAL) || (vertical < -CAMERAVERTICAL) ||
            (horizontal>CAMERAHORIZONTAL) || (horizontal<-CAMERAHORIZONTAL))
        {
            vertical   -= dv;
            horizontal -= dh;
            return false;
        }

        double x = CAMERARADIUS * Math.cos(vertical) * Math.cos(horizontal);
        double y = CAMERARADIUS * Math.cos(vertical) * Math.sin(horizontal);
        double z = CAMERARADIUS * Math.sin(vertical) + CAMERAHEIGHT;

        return (output.updateCamera(new Vector3D(x, y, z),
                                    new Vector3D(0.0, 0.0, CAMERAHEIGHT),
                                    new Vector3D(0.0, 0.0, 1.0)));
    }

    /**
    * Default constructor
    * purpose : initialize the output window and set its size to 300x300
    */
    public ArmDisplay()
    {
        this(new String(""), DEFAULTUSIZE, DEFAULTVSIZE);
    }

    /**
    * Constructor
    * purpose : initialize the output window
    * @param u horizontal size of the output area
    * @param v vertical size of the output area
    */
    public ArmDisplay(int u, int v)
    {
        this(new String(""), u, v);
    }

    /**
    * Constructor
    * purpose : initialize the output window
    * @param title title of the window
    * @param u horizontal size of the output area
    * @param v vertical size of the output area
    */
    public ArmDisplay(String title, int u, int v)
    {
        /* initialize the output window */
 	System.out.println("Arm Display starts");
        setTitle(title);
        int usize = ((u < DEFAULTUSIZE) ? DEFAULTUSIZE : u);
        int vsize = ((v < DEFAULTVSIZE) ? DEFAULTVSIZE : v);
        setSize(usize, vsize + BUTTONHEIGHT);
        setLayout(null);

        output = new Pic3DCanvas(usize, vsize);
        output.setBounds(0, 0, usize, vsize);
        add("Output", output);

        /* create all buttons */
        button1 = new Button("Tilt Up");
        button1.addActionListener(new B1());
        button1.setBounds(0, vsize, BUTTONWIDTH, BUTTONHEIGHT);
        add(button1);
        button2 = new Button("Tilt Down");
        button2.addActionListener(new B2());
        button2.setBounds(BUTTONWIDTH, vsize, BUTTONWIDTH, BUTTONHEIGHT);
        add(button2);
        button3 = new Button("Spin Left");
        button3.addActionListener(new B3());
        button3.setBounds(2*BUTTONWIDTH, vsize, BUTTONWIDTH, BUTTONHEIGHT);
        add(button3);
        button4 = new Button("Spin Right");
        button4.addActionListener(new B4());
        button4.setBounds(3*BUTTONWIDTH, vsize, BUTTONWIDTH, BUTTONHEIGHT);
        add(button4);
        button5 = new Button("Quit");
        button5.addActionListener(new B5());
         button5.setBounds(usize-BUTTONWIDTH, vsize, BUTTONWIDTH, BUTTONHEIGHT);
        add(button5);

        /* set the camera */
        vertical   = 0.0;
        horizontal = 0.0;
        output.updateCamera(new Vector3D(CAMERARADIUS, 0.0, CAMERAHEIGHT),
                            new Vector3D(0.0, 0.0, CAMERAHEIGHT),
                            new Vector3D(0.0, 0.0, 1.0));

        /* bring the output window to foreground */
        setVisible(true);
	validate();
 	System.out.println("Arm Display constructor completes");
    }

    /**
    * purpose : add a new Object3D into the scene
    * @param x Object3D
    * @return index of the input Object3D
    */
    public int addObject3D(Object3D x)
    {
        return (output.addObject3D(x));
    }

    /**
    * purpose : change the current transformational matrix of an object
    * @param index index of the object
    * @param m new current transformational matrix
    * @return true (succeed) or false (fail)
    */
    public boolean changeCTM(int index, Matrix m)
    {
        return (output.changeCTM(index, m));
    }

    /**
    * purpose : show the scene on the output window
    */
    public void display()
    {
        output.display();
    }

    /**
    * purpose : get the total number of objects
    * @return total number of objects
    */
    public int numberOfObjects()
    {
        return (output.numberOfObjects());
    }

    /**
    * purpose : update the frame
    */
    public void update(Graphics g)
    {
        paint(g);
    }
};
