import java.awt.*;
import java.io.*;

/**
* A class to simulate the action of the robot arm
* @author Ho Ng
* @version 01/09/1998
*/
public class ArmSimulator extends CRSPlusA150
{
    private static final String FILENAME  = "arminput.dat";
    private static final int DEFAULTSIZE  = 512;
    private static final int FRAMESPERSEC = 20;
    private static final int TIMEGAP      = 1000 / FRAMESPERSEC;
    private static final int EMPTYMODE    = 0;
    private static final int WINDOWMODE   = 1;
    private static final int CANVASMODE   = 2;

    private ArmDisplay  window;
    private Pic3DCanvas canvas;
    private int displayMode;

    /**
    * purpose : load the object's structure from the data file
    * @return true (succeed) or false (fail)
    */
    private boolean loadObjectStructure()
    {
        Object3D obj;

        try
        {
            /* open the input file */
            FileInputStream fis = new FileInputStream(FILENAME);
            BufferedReader dis = new BufferedReader(new InputStreamReader(fis));

            /* load data and create the structure */
            for (int i = 0; i <= 7; i++)
            {
                obj = new Object3D(dis);
                if (displayMode == WINDOWMODE)
                    window.addObject3D(obj);
                else
                    canvas.addObject3D(obj);
            }

            /* close the input file */
            fis.close();
            return true;
        }
        catch (Exception e)
        {
            System.out.println("ArmSimulator : Could not model the scene " +
                               e);
            return false;
        }
    }    

    /**
    * purpose : update all CTMs of the objects in the scene and display it
    */
    private void updateCTM()
    {
        if (displayMode == WINDOWMODE)
        {
            for (int i = 1; i <= 7; i++)
                window.changeCTM(i, getTransformation(i));
            window.display();
        }
        else if (displayMode == CANVASMODE)
        {
            for (int i = 1; i <= 7; i++)
                canvas.changeCTM(i, getTransformation(i));
            canvas.display();
        }
    }

    /**
    * purpose : increment all joint angles
    * @param waist increment waist angle in radian
    * @param shoulder increment shoulder angle in radian
    * @param elbow increment elbow angle in radian
    * @param wrist increment wrist's bend angle in radian
    * @param twist increment wrist's twist angle in radian
    * @return true (succeed) or false (fail)
    */
    private boolean jointsMotion(double waist, double shoulder, double elbow,
                                 double wrist, double twist)
    {
        double[] angle   = {getTheta(1), getTheta(2), getTheta(3), getTheta(4),
                            getTheta(5)};

        double dwaist    = waist / FRAMESPERSEC;
        double dshoulder = shoulder / FRAMESPERSEC;
        double delbow    = elbow / FRAMESPERSEC;
        double dwrist    = wrist / FRAMESPERSEC;
        double dtwist    = twist / FRAMESPERSEC;

        for (int i = 0; i < FRAMESPERSEC; i++)
        {
            if (super.mi(dwaist, dshoulder, delbow, dwrist, dtwist) == false)
            {
                /* move back to original position */
                super.ma(angle[0], angle[1], angle[2], angle[3], angle[4]);
                updateCTM();
                return false;
            }

            updateCTM();
            try
            {
                Thread.sleep(TIMEGAP);
            }
            catch (Exception e)
            {
                // do nothing
            }
        }

        return true;
    }

    /**
    * purpose : move the fingers of the gripper
    * @param value width between two fingers in inch
    * @return true (succeed) or false (fail)
    */
    private boolean gripperMotion(double value)
    {
        double ovalue = getD(6) + getD(7);
        double cvalue = ovalue;
        double dvalue = (value - ovalue) / FRAMESPERSEC;

        for (int i = 0; i < FRAMESPERSEC; i++)
        {
            cvalue += dvalue;
            if (super.grip(cvalue) == false)
            {
                /* move back to original position */
                super.grip(ovalue);
                updateCTM();
                return false;
            }

            updateCTM();
            try
            {
                Thread.sleep(TIMEGAP);
            }
            catch (Exception e)
            {
                // do nothing
            }
        }

        return true;
    }

    /**
    * Default constructor
    * purpose : initialize the simulator
    */
    public ArmSimulator()
    {
        super();
        displayMode = EMPTYMODE;
    }

    /**
    * purpose : create an output window
    * @return true (succeed) or false (fail)
    */
    public boolean openWindow()
    {
        return openWindow(DEFAULTSIZE, DEFAULTSIZE);
    }

    /**
    * purpose : create an output window
    * @return true (succeed) or false (fail)
    */
    public boolean openWindow(int usize, int vsize)
    {
        if (displayMode != EMPTYMODE)
            return false;

        displayMode = WINDOWMODE;
        window = new ArmDisplay(new String("CRSPlusA150"), usize, vsize);
        loadObjectStructure();
        updateCTM();
        return true;
    }

    /**
    * get the output canvas
    * @param usize total number of pixels of the output canvas horizontally
    * @param vsize total number of pixels of the output canvas vertically
    * @return output Pic3DCanvas
    */
    public Pic3DCanvas getCanvas(int usize, int vsize)
    {
        if (displayMode != EMPTYMODE)
            return null;

        displayMode = CANVASMODE;
        canvas = new Pic3DCanvas(usize, vsize);
        loadObjectStructure();
        updateCTM();
        return canvas;
    }

    /**
    * purpose : home the arm
    */
    public void home()
    {
        super.home();
    }

    /**
    * purpose : set the arm to ready position
    */
    public void ready()
    {
        super.ready();
        updateCTM();
    }

    /**
    * purpose : execute the command MA of the arm
    * @param waist waist angle in radian
    * @param shoulder shoulder angle in radian
    * @param elbow elbow angle in radian
    * @param wrist wrist's bend angle in radian
    * @param twist wrist's twist angle in radian
    * @return true (succeed) or false (fail)
    */
    public boolean ma(double waist, double shoulder, double elbow,
                      double wrist, double twist)
    {
        return jointsMotion(waist - getTheta(1), shoulder - getTheta(2),
                            elbow - getTheta(3), wrist - getTheta(4),
                            twist - getTheta(5));
    }

    /**
    * purpose : execute the command MA of the arm
    * @param waist waist angle in degree
    * @param shoulder shoulder angle in degree
    * @param elbow elbow angle in degree
    * @param wrist wrist's bend angle in degree
    * @param twist wrist's twist angle in degree
    * @return true (succeed) or false (fail)
    */
    public boolean madeg(double waist, double shoulder, double elbow,
                         double wrist, double twist)
    {
        return jointsMotion(degToRad(waist) - getTheta(1),
                            degToRad(shoulder) - getTheta(2),
                            degToRad(elbow) - getTheta(3),
                            degToRad(wrist) - getTheta(4),
                            degToRad(twist) - getTheta(5));
    }

    /**
    * purpose : execute command MI of the arm
    * @param waist increment of waist angle in radian
    * @param shoulder increment of shoulder angle in radian
    * @param elbow increment of elbow angle in radian
    * @param wrist increment of wrist's bend angle in radian
    * @param twist increment of wrist's twist angle in radian
    * @return true (succeed) or false (fail)
    */
    public boolean mi(double waist, double shoulder, double elbow,
                      double wrist, double twist)
    {
        return jointsMotion(waist, shoulder, elbow, wrist, twist);
    }

    /**
    * purpose : execute command MI of the arm
    * @param waist increment of waist angle in degree
    * @param shoulder increment of shoulder angle in degree
    * @param elbow increment of elbow angle in degree
    * @param wrist increment of wrist's bend angle in degree
    * @param twist increment of wrist's twist angle in degree
    * @return true (succeed) or false (fail)
    */
    public boolean mideg(double waist, double shoulder, double elbow,
                         double wrist, double twist)
    {
        return jointsMotion(degToRad(waist), degToRad(shoulder),
                            degToRad(elbow), degToRad(wrist), degToRad(twist));
    }

    /**
    * purpose : execute the command GRIP of the arm
    * @param width width between the fingers of the gripper in inch
    * @return true (succeed) or false (fail)
    */
    public boolean grip(double width)
    {
        return gripperMotion(width);
    }

    /**
    * purpose : execute the command SPEED of the arm
    * @param value speed of the arm
    * @return true (succeed) or false (fail)
    */
    public boolean speed(int value)
    {
        return (super.speed(value));
    }
};
