import java.io.*;

/**
* A class to represent a 4x4 transformational matrix
* @author Ho Ng
* @version 01/09/1998
*/
public class Matrix
{
    private static final int DIMENSION = 4;

    /**
    * double m[4][4] <br>
    * purpose : store the values of the matrix
    */
    public double[][] m = new double[DIMENSION][DIMENSION];

    /**
    * Default constructor <br>
    * purpose : initialize the matrix to a zero matrix
    */
    public Matrix()
    {
    }

    /**
    * Copy constructor <br>
    * purpose : initialize the matrix to the input Matrix x
    * @param x Matrix
    */
    public Matrix(Matrix x)
    {
        for (int i = 0; i < DIMENSION; i++)
            for (int j = 0; j < DIMENSION; j++)
                m[i][j] = x.m[i][j];
    }

    /**
    * purpose : create an identity matrix
    * @return an identity matrix
    */
    public static Matrix identity()
    {
        Matrix result = new Matrix();

        result.m[0][0] = 1.0;
        result.m[1][1] = 1.0;
        result.m[2][2] = 1.0;
        result.m[3][3] = 1.0;

        return result;
    }

    /**
    * purpose : add two matrices
    * @param a first Matrix
    * @param b second Matrix
    * @return a new copy of a + b
    */
    public static Matrix plus(Matrix a, Matrix b)
    {
        Matrix result = new Matrix();
        int r, c;

        for (r = 0; r < DIMENSION; r++)
            for (c = 0; c < DIMENSION; c++)
                result.m[r][c] = (a.m[r][c] + b.m[r][c]);

        return result;
    }

    /**
    * purpose : subtract one matrix by another matrix
    * @param a first Matrix
    * @param b second Matrix
    * @return a new copy of a - b
    */
    public static Matrix minus(Matrix a, Matrix b)
    {
        Matrix result = new Matrix();
        int r, c;

        for (r = 0; r < DIMENSION; r++)
            for (c = 0; c < DIMENSION; c++)
                result.m[r][c] = (a.m[r][c] - b.m[r][c]);

        return result;
    }

    /**
    * purpose : multiply two matrices
    * @param a first Matrix
    * @param b second Matrix
    * @return a new copy of a x b
    */
    public static Matrix multi(Matrix a, Matrix b)
    {
        Matrix result = new Matrix();
        int r, c, i;

        for (r = 0; r < DIMENSION; r++)
            for (c = 0; c < DIMENSION; c++)
                for (i = 0; i < DIMENSION; i++)
                    result.m[r][c] += (a.m[r][i] * b.m[i][c]);

        return result;
    }

    /**
    * purpose : create a translational matrix
    * @param tx translate distance along x-axis
    * @param ty translate distance along y-axis
    * @param tz translate distance along z-axis
    * @return a translational matrix
    */
    public static Matrix translate(double tx, double ty, double tz)
    {
        Matrix temp = Matrix.identity();

        temp.m[0][3] = tx;
        temp.m[1][3] = ty;
        temp.m[2][3] = tz;

        return temp;
    }

    /**
    * purpose : create a scale matrix
    * @param sx scale factor along x-axis
    * @param sy scale factor along y-axis
    * @param sz scale factor along z-axis
    * @return a scale matrix
    */
    public static Matrix scale(double sx, double sy, double sz)
    {
        Matrix temp = new Matrix();

        temp.m[0][0] = sx;
        temp.m[1][1] = sy;
        temp.m[2][2] = sz;
        temp.m[3][3] = 1.0;

        return temp;
    }

    /**
    * purpose : create a rotational matrix
    * @param axis rotation axis (x, y or z)
    * @param angle rotate by angle in radian
    * @return a rotational matrix
    */
    public static Matrix rotate(char axis, double angle)
    {
        Matrix temp = Matrix.identity();

        if (axis == 'x')
        {
            temp.m[1][1] = Math.cos(angle);
            temp.m[1][2] = -Math.sin(angle);
            temp.m[2][1] = Math.sin(angle);
            temp.m[2][2] = Math.cos(angle);
        }
        else if (axis == 'y')
            {
            temp.m[0][0] = Math.cos(angle);
            temp.m[0][2] = Math.sin(angle);
            temp.m[2][0] = -Math.sin(angle);
            temp.m[2][2] = Math.cos(angle);
        }
        else if (axis == 'z')
        {
            temp.m[0][0] = Math.cos(angle);
            temp.m[0][1] = -Math.sin(angle);
            temp.m[1][0] = Math.sin(angle);
            temp.m[1][1] = Math.cos(angle);
        }
        else
            System.out.println("Matrix.rotate : axis should be x, y or z");

        return temp;
    }

    /**
    * purpose : print the matrix
    */
    public void print()
    {
        for (int i = 0; i < DIMENSION; i++)
        {
            for (int j = 0; j < DIMENSION; j++)
                System.out.print(m[i][j] + " ");
            System.out.print("\n");
        }
    }
};
