import java.io.*;

/**
* A class to represent a vector or a point in 3D system
* @author Ho Ng
* @version 01/09/1998
*/
public class Vector3D
{
    /**
    * double v[3] <br>
    * purpose  : store the values of the coordinates
    */
    public double[] v = new double[3];

    /**
    * Default constructor <br>
    * purpose : initialize the vector to (0.0, 0.0, 0.0)
    */
    public Vector3D()
    {
    }

    /**
    * Constructor <br>
    * purpose : initialize the vector to (x, y, z)
    * @param x x-coordinate
    * @param y y-coordinate
    * @param z z-coordinate
    */
    public Vector3D(double x, double y, double z)
    {
        v[0] = x;
        v[1] = y;
        v[2] = z;
    }

    /**
    * Copy constructor <br>
    * purpose : initialize the vector to the input Vector3D x)
    */
    public Vector3D(Vector3D x)
    {
        v[0] = x.v[0];
        v[1] = x.v[1];
        v[2] = x.v[2];
    }

    /**
    * purpose add two vectors together
    * @param a first Vector3D
    * @param b second Vector3D
    * @return a new copy of a + b
    */
    public static Vector3D plus(Vector3D a, Vector3D b)
    {
        return (new Vector3D(a.v[0] + b.v[0], a.v[1] + b.v[1],
                             a.v[2] + b.v[2]));
    }

    /**
    * purpose : subtract one vector by another vector
    * @param a first Vector3D
    * @param b second Vector3D
    * @return a new copy of a - b
    */
    public static Vector3D minus(Vector3D a, Vector3D b)
    {
        return (new Vector3D(a.v[0] - b.v[0], a.v[1] - b.v[1],
                             a.v[2] - b.v[2]));
    }

    /**
    * purpose : multiply a number with a vector
    * @param factor integer
    * @param a Vector3D
    * @return a new copy of factor * a
    */
    public static Vector3D multi(int factor, Vector3D a)
    {
        return (new Vector3D(factor * a.v[0], factor * a.v[1], factor * a.v[2]));
    }

    /**
    * purpose : dot product
    * @param a first Vector3D
    * @param b second Vector3D
    * @return a(.)b
    */
    public static double dot(Vector3D a, Vector3D b)
    {
        return (a.v[0] * b.v[0] + a.v[1] * b.v[1] + a.v[2] * b.v[2]);
    }

    /**
    * purpose : cross product
    * @param a first Vector3D
    * @param b second Vector3D
    * @return a(x)b
    */
    public static Vector3D cross(Vector3D a, Vector3D b)
    {
        Vector3D result = new Vector3D();

        result.v[0] = a.v[1] * b.v[2] - a.v[2] * b.v[1];
        result.v[1] = a.v[2] * b.v[0] - a.v[0] * b.v[2];
        result.v[2] = a.v[0] * b.v[1] - a.v[1] * b.v[0];

        return result;
    }

    /**
    * purpose : length of the vector
    * @return length of the Vector3D
    */
    public double length()
    {
        return (Math.sqrt(dot(this, this)));
    }

    /**
    * purpose : normalizing the vector
    * @return length of the original Vector3D
    */
    public double normalize()
    {
        double d = length();

        if (d != 0.0)
        {
            v[0] /= d;
            v[1] /= d;
            v[2] /= d;
        }

        return d;
    }

    /**
    * purpose : multiply a matrix with a vector, assume last row of the matrix is 0,0,0,1
    * @param m Matrix
    * @param p Vector3D
    * @return a new copy of m x p
    */
    public static Vector3D transformPointQ(Matrix m, Vector3D p)
    {
        Vector3D result = new Vector3D();
 
        result.v[0] = m.m[0][0] * p.v[0] + m.m[0][1] * p.v[1] +
                      m.m[0][2] * p.v[2] + m.m[0][3];
        result.v[1] = m.m[1][0] * p.v[0] + m.m[1][1] * p.v[1] +
                      m.m[1][2] * p.v[2] + m.m[1][3];
        result.v[2] = m.m[2][0] * p.v[0] + m.m[2][1] * p.v[1] +
                      m.m[2][2] * p.v[2] + m.m[2][3];
 
        return result;
    }

    /**
    * purpose : multiply a matrix with a vector
    * @param m Matrix
    * @param p Vector3D
    * @return a new copy of m x p
    */
    public static Vector3D transformPoint(Matrix m, Vector3D p)
    {
        Vector3D result = Vector3D.transformPointQ(m, p);

        double w = m.m[3][0] * p.v[0] + m.m[3][1] * p.v[1] +
                   m.m[3][2] * p.v[2] + m.m[3][3];
 
        /* assume w != 0.0 */
        result.v[0] /= w;
        result.v[1] /= w;
        result.v[2] /= w;
 
        return result;
    }

    /**
    * purpose : convert the vector to String
    * @return the vector in String format
    */
    public String toString()
    {
        return ("(" + v[0] + ", " + v[1] + ", " + v[2] + ")");
    }

    /**
    * purpose : round off the coordinates of the vector to some decimal places
    * @param decPlace number of decimal places
    */
    public void round(int decPlace)
    {
        int factor = (int)(Math.pow(10, decPlace));

        v[0] = (double)(Math.round(v[0] * factor)) / factor;
        v[1] = (double)(Math.round(v[1] * factor)) / factor;
        v[2] = (double)(Math.round(v[2] * factor)) / factor;
    }
};
